@@ -409,8 +409,9 @@ class TypeMarshaller(object):
 
         # The metadata that is written depends on the format.
 
+        dsetgrp = grp[name]
         if options.store_python_metadata:
-            set_attribute_string(grp[name], 'Python.Type', type_string)
+            set_attribute_string(dsetgrp, 'Python.Type', type_string)
 
         # If we are not storing python information or doing MATLAB
         # compatibility, then attributes not in the python and/or
@@ -424,8 +425,8 @@ class TypeMarshaller(object):
         if options.matlab_compatible:
             attributes_used |= self.matlab_attributes
 
-        for attribute in (set(grp[name].attrs.keys()) - attributes_used):
-            del_attribute(grp[name], attribute)
+        for attribute in (set(dsetgrp.attrs.keys()) - attributes_used):
+            del_attribute(dsetgrp, attribute)
 
     def read(self, f, grp, name, options):
         """ Read a Python object from file.
@@ -688,7 +689,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
         #
         # A flag, wrote_as_struct, is set depending on which path is
         # taken, which is then passed onto write_metadata.
-        
+
         if data_to_store.dtype.fields is not None \
                 and h5py.check_dtype(ref=data_to_store.dtype) \
                 is not h5py.Reference \
@@ -724,7 +725,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
             # Write the metadata, and set the MATLAB_class to 'struct'
             # explicitly.
             if options.matlab_compatible:
-                set_attribute_string(grp[name], 'MATLAB_class',
+                set_attribute_string(grp2, 'MATLAB_class',
                                      'struct')
 
             # Delete any Datasets/Groups not corresponding to a field
@@ -742,6 +743,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
             # Dataset as opposed to a HDF5 Reference array). The H5PATH
             # attribute needs to be set appropriately, while all other
             # attributes need to be deleted.
+            grp2_name = grp2.name
             for field in field_names:
                 new_data = np.zeros(shape=data_to_store.shape,
                                     dtype='object')
@@ -765,20 +767,21 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
                     write_data(f, grp2, field, new_data, None, options)
 
                 if field in grp2:
+                    grp2_field = grp2[field]
                     if options.matlab_compatible:
-                        set_attribute_string(grp2[field], 'H5PATH',
-                                             grp2.name)
+                        set_attribute_string(grp2_field, 'H5PATH',
+                                             grp2_name)
                     else:
-                        del_attribute(grp2[field], 'H5PATH')
+                        del_attribute(grp2_field, 'H5PATH')
 
                     # In the case that we wrote a Reference array (not a
                     # single element), then all other attributes need to
                     # be removed.
                     if np.prod(new_data.shape) != 1:
                         for attribute in (set( \
-                                grp2[field].attrs.keys()) \
+                                grp2_field.attrs.keys()) \
                                 - set(['H5PATH'])):
-                            del_attribute(grp2[field], attribute)
+                            del_attribute(grp2_field, attribute)
         else:
             wrote_as_struct = False
             # If it has fields and it isn't a Reference type, none of
@@ -841,19 +844,23 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
             if name not in grp:
                 grp.create_dataset(name, data=data_to_store,
                                    **filters)
-            elif not isinstance(grp[name], h5py.Dataset) \
-                    or grp[name].dtype != data_to_store.dtype \
-                    or grp[name].shape != data_to_store.shape \
-                    or grp[name].compression != filters['compression'] \
-                    or grp[name].shuffle != filters['shuffle'] \
-                    or grp[name].fletcher32 != filters['fletcher32'] \
-                    or grp[name].compression_opts != \
-                    filters['compression_opts']:
-                del grp[name]
-                grp.create_dataset(name, data=data_to_store,
-                                   **filters)
             else:
-                grp[name][...] = data_to_store
+                # avoid multiple calls to __getitem__ by storing the
+                # reference in a local variable
+                dset = grp[name]
+                if not isinstance(dset, h5py.Dataset) \
+                        or dset.dtype != data_to_store.dtype \
+                        or dset.shape != data_to_store.shape \
+                        or dset.compression != filters['compression'] \
+                        or dset.shuffle != filters['shuffle'] \
+                        or dset.fletcher32 != filters['fletcher32'] \
+                        or dset.compression_opts != \
+                        filters['compression_opts']:
+                    del grp[name]
+                    grp.create_dataset(name, data=data_to_store,
+                                       **filters)
+                else:
+                    dset[...] = data_to_store
 
         # Write the metadata using the inherited function (good enough).
         self.write_metadata(f, grp, name, data, type_string,
@@ -877,8 +884,12 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
         # numpy type, and its type of container ('scalar', 'ndarray',
         # 'matrix', or 'chararray') need to be stored.
 
+        # avoid multiple calls to __getitem__ by storing the
+        # reference in a local variable
+        dsetgrp = grp[name]
+
         if options.store_python_metadata:
-            set_attribute(grp[name], 'Python.Shape',
+            set_attribute(dsetgrp, 'Python.Shape',
                           np.uint64(data.shape))
 
             # Now, in Python 3, the dtype names for bare bytes and
@@ -886,7 +897,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
             # but in Python 2, they start with 'string' and 'unicode'
             # respectively. The Python 2 ones must be converted to the
             # Python 3 ones for writing.
-            set_attribute_string(grp[name], \
+            set_attribute_string(dsetgrp, \
                 'Python.numpy.UnderlyingType', \
                 data.dtype.name.replace('string', 'bytes').replace( \
                 'unicode', 'str'))
@@ -900,7 +911,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
                 container = 'ndarray'
             else:
                 container = 'scalar'
-            set_attribute_string(grp[name], 'Python.numpy.Container',
+            set_attribute_string(dsetgrp, 'Python.numpy.Container',
                                  container)
 
         # If it was written like a matlab struct, then we set the
@@ -919,11 +930,11 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
 
             # Write or delete 'Python.Fields' as appropriate.
             if options.store_python_metadata:
-                set_attribute_string_array(grp[name],
+                set_attribute_string_array(dsetgrp,
                                            'Python.Fields',
                                            field_names)
             else:
-                del_attribute(grp[name], 'Python.Fields')
+                del_attribute(dsetgrp, 'Python.Fields')
 
             # If we are making it MATLAB compatible and have h5py
             # version >= 2.3, then we can set the MATLAB_fields
@@ -942,14 +953,14 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
                         fs[i] = np.array([c.encode('ascii') for c in s],
                                          dtype='S1')
                 except UnicodeEncodeError:
-                    del_attribute(grp[name], 'MATLAB_fields')
+                    del_attribute(dsetgrp, 'MATLAB_fields')
                 else:
-                    set_attribute(grp[name], 'MATLAB_fields', fs)
+                    set_attribute(dsetgrp, 'MATLAB_fields', fs)
             else:
-                del_attribute(grp[name], 'MATLAB_fields')
+                del_attribute(dsetgrp, 'MATLAB_fields')
         else:
-            del_attribute(grp[name], 'Python.Fields')
-            del_attribute(grp[name], 'MATLAB_fields')
+            del_attribute(dsetgrp, 'Python.Fields')
+            del_attribute(dsetgrp, 'MATLAB_fields')
 
         # If data is empty, we need to set the Python.Empty and
         # MATLAB_empty attributes to 1 if we are storing type info or
@@ -960,18 +971,18 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
                 or data.dtype.type == np.str_)
                 and data.nbytes == 0):
             if options.store_python_metadata:
-                set_attribute(grp[name], 'Python.Empty',
-                                          np.uint8(1))
+                set_attribute(dsetgrp, 'Python.Empty',
+                              np.uint8(1))
             else:
-                del_attribute(grp[name], 'Python.Empty')
+                del_attribute(dsetgrp, 'Python.Empty')
             if options.matlab_compatible:
-                set_attribute(grp[name], 'MATLAB_empty',
-                                          np.uint8(1))
+                set_attribute(dsetgrp, 'MATLAB_empty',
+                              np.uint8(1))
             else:
-                del_attribute(grp[name], 'MATLAB_empty')
+                del_attribute(dsetgrp, 'MATLAB_empty')
         else:
-            del_attribute(grp[name], 'Python.Empty')
-            del_attribute(grp[name], 'MATLAB_empty')
+            del_attribute(dsetgrp, 'Python.Empty')
+            del_attribute(dsetgrp, 'MATLAB_empty')
 
         # If we are making it MATLAB compatible, the MATLAB_class
         # attribute needs to be set looking up the data type (gotten
@@ -986,24 +997,24 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
         if options.matlab_compatible:
             if data.dtype.fields is not None \
                     and options.structured_numpy_ndarray_as_struct:
-                set_attribute_string(grp[name], 'MATLAB_class',
+                set_attribute_string(dsetgrp, 'MATLAB_class',
                                      'struct')
             elif tp in self.__MATLAB_classes:
-                set_attribute_string(grp[name], 'MATLAB_class',
+                set_attribute_string(dsetgrp, 'MATLAB_class',
                                      self.__MATLAB_classes[tp])
                 if tp in (np.bytes_, np.str_, np.bool_):
-                    set_attribute(grp[name], 'MATLAB_int_decode',
+                    set_attribute(dsetgrp, 'MATLAB_int_decode',
                                   np.int64(grp[name].dtype.itemsize))
                 else:
-                    del_attribute(grp[name], 'MATLAB_int_decode')
+                    del_attribute(dsetgrp, 'MATLAB_int_decode')
             else:
-                del_attribute(grp[name], 'MATLAB_class')
-                del_attribute(grp[name], 'MATLAB_empty')
-                del_attribute(grp[name], 'MATLAB_int_decode')
+                del_attribute(dsetgrp, 'MATLAB_class')
+                del_attribute(dsetgrp, 'MATLAB_empty')
+                del_attribute(dsetgrp, 'MATLAB_int_decode')
         else:
-            del_attribute(grp[name], 'MATLAB_class')
-            del_attribute(grp[name], 'MATLAB_empty')
-            del_attribute(grp[name], 'MATLAB_int_decode')
+            del_attribute(dsetgrp, 'MATLAB_class')
+            del_attribute(dsetgrp, 'MATLAB_empty')
+            del_attribute(dsetgrp, 'MATLAB_int_decode')
 
     def read(self, f, grp, name, options):
         # If name is not present, then we can't read it and have to
@@ -1011,6 +1022,9 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
         if name not in grp:
             raise NotImplementedError(name + ' is not present.')
 
+        # Get the object.
+        dsetgrp = grp[name]
+
         # Get the different attributes this marshaller uses.
 
         if sys.hexversion >= 0x03000000:
@@ -1018,7 +1032,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
         else:
             defaultfactory = lambda : None
         attributes = collections.defaultdict(defaultfactory)
-        read_all_attributes_into(grp[name].attrs, attributes)
+        read_all_attributes_into(dsetgrp.attrs, attributes)
 
         str_attrs = dict()
         for attr_name in ('Python.Type',
@@ -1064,19 +1078,19 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
         # recursively). If it is a Group, then it is a structured
         # ndarray like object that needs to be read field wise and
         # constructed.
-        if isinstance(grp[name], h5py.Dataset):
+        if isinstance(dsetgrp, h5py.Dataset):
             # Read the data.
-            data = grp[name][...]
+            data = dsetgrp[...]
 
             # If it is a reference type, then we need to make an object
             # array that is its replicate, but with the objects they are
             # pointing to in their elements instead of just the
             # references.
-            if h5py.check_dtype(ref=grp[name].dtype) is not None:
+            if h5py.check_dtype(ref=dsetgrp.dtype) is not None:
                 data = read_object_array(f, data, options)
         else:
             # Starting with an empty dict, all that has to be done is
-            # iterate through all the Datasets and Groups in grp[name]
+            # iterate through all the Datasets and Groups in dsetgrp
             # and add them to a dict with their name as the key. Since
             # we don't want an exception thrown by reading an element to
             # stop the whole reading process, the reading is wrapped in
@@ -1092,11 +1106,10 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
             # struct_data.
             struct_data = dict()
             is_multi_element = True
-            for k in grp[name]:
+            for k, fld in dsetgrp.items():
                 # We must exclude group_for_references
-                if grp[name][k].name == options.group_for_references:
+                if fld.name == options.group_for_references:
                     continue
-                fld = grp[name][k]
                 if isinstance(fld, h5py.Group) \
                         or h5py.check_dtype(ref=fld.dtype) is None \
                         or len(set(fld.attrs.keys()) \
@@ -1106,7 +1119,7 @@ class NumpyScalarArrayMarshaller(TypeMarshaller):
                         'Python.Empty']))) != 0:
                     is_multi_element = False
                 try:
-                    struct_data[k] = read_data(f, grp[name], k,
+                    struct_data[k] = read_data(f, dsetgrp, k,
                                                options)
                 except:
                     pass
@@ -1604,12 +1617,12 @@ class PythonDictMarshaller(TypeMarshaller):
         # before being created.
 
         if name not in grp:
-            grp.create_group(name)
+            grp2 = grp.create_group(name)
         elif not isinstance(grp[name], h5py.Group):
             del grp[name]
-            grp.create_group(name)
-
-        grp2 = grp[name]
+            grp2 = grp.create_group(name)
+        else:
+            grp2 = grp[name]
 
         # Write the metadata.
         self.write_metadata(f, grp, name, data, type_string, options)
@@ -1626,19 +1639,22 @@ class PythonDictMarshaller(TypeMarshaller):
         # needs to be set as the path of grp2 on all of them if we are
         # doing MATLAB compatibility (otherwise, the attribute needs to
         # be deleted).
+        grp2_name = grp2.name
         for k, v in data.items():
             write_data(f, grp2, k, v, None, options)
             if k in grp2:
                 if options.matlab_compatible:
-                    set_attribute_string(grp2[k], 'H5PATH', grp2.name)
+                    set_attribute_string(grp2[k], 'H5PATH', grp2_name)
                 else:
                     del_attribute(grp2[k], 'H5PATH')
 
     def write_metadata(self, f, grp, name, data, type_string, options):
-        # First, call the inherited version to do most of the work.
+        # First, call the inherited version to do most of the work and
+        # get the group.
 
         TypeMarshaller.write_metadata(self, f, grp, name, data,
                                       type_string, options)
+        grp2 = grp[name]
 
         # Grab all the keys and sort the list.
         fields = sorted(list(data.keys()))
@@ -1646,7 +1662,7 @@ class PythonDictMarshaller(TypeMarshaller):
         # If we are storing python metadata, we need to set the
         # 'Python.Fields' Attribute to be all the keys.
         if options.store_python_metadata:
-            set_attribute_string_array(grp[name], 'Python.Fields',
+            set_attribute_string_array(grp2, 'Python.Fields',
                                        fields)
 
         # If we are making it MATLAB compatible and have h5py version
@@ -1664,11 +1680,11 @@ class PythonDictMarshaller(TypeMarshaller):
                     fs[i] = np.array([c.encode('ascii') for c in s],
                                      dtype='S1')
             except UnicodeDecodeError:
-                del_attribute(grp[name], 'MATLAB_fields')
+                del_attribute(grp2, 'MATLAB_fields')
             else:
-                set_attribute(grp[name], 'MATLAB_fields', fs)
+                set_attribute(grp2, 'MATLAB_fields', fs)
         else:
-            del_attribute(grp[name], 'MATLAB_fields')
+            del_attribute(grp2, 'MATLAB_fields')
 
         # If we are making it MATLAB compatible, the MATLAB_class
         # attribute needs to be set for the data type. If the type
@@ -1677,17 +1693,20 @@ class PythonDictMarshaller(TypeMarshaller):
 
         tp = type(data)
         if options.matlab_compatible and tp in self.__MATLAB_classes:
-            set_attribute_string(grp[name], 'MATLAB_class',
+            set_attribute_string(grp2, 'MATLAB_class',
                                  self.__MATLAB_classes[tp])
         else:
-            del_attribute(grp[name], 'MATLAB_class')
+            del_attribute(grp2, 'MATLAB_class')
 
     def read(self, f, grp, name, options):
         # If name is not present or is not a Group, then we can't read
         # it and have to throw an error.
-        if name not in grp or not isinstance(grp[name], h5py.Group):
-            raise NotImplementedError('No Group ' + name +
-                                      ' is present.')
+        grp2 = grp.get(name)
+        if grp2 is None:
+            raise NotImplementedError('No object with name ' + name
+                                      + 'is present.')
+        if not isinstance(grp2, h5py.Group):
+            raise NotImplementedError(name + ' is not a Group.')
 
         # Starting with an empty dict, all that has to be done is
         # iterate through all the Datasets and Groups in grp[name] and
@@ -1697,12 +1716,12 @@ class PythonDictMarshaller(TypeMarshaller):
         # block that just catches exceptions and then does nothing about
         # them (nothing needs to be done).
         data = dict()
-        for k in grp[name]:
+        for k, dsetgrp in grp2.items():
             # We must exclude group_for_references
-            if grp[name][k].name == options.group_for_references:
+            if dsetgrp.name == options.group_for_references:
                 continue
             try:
-                data[k] = read_data(f, grp[name], k, options)
+                data[k] = read_data(f, grp2, k, options)
             except:
                 pass
         return data
