@@ -28,11 +28,11 @@ This is the hdf5storage package, a Python package to read and write
 python data types to HDF5 (Heirarchal Data Format) files beyond just
 Numpy types.
 
-Version 0.1.15
+Version 0.1.18
 
 """
 
-__version__ = "0.1.15"
+__version__ = "0.1.18"
 
 import sys
 import os
@@ -441,7 +441,7 @@ class Options(object):
 
     @property
     def convert_numpy_bytes_to_utf16(self):
-        """ Whether or not to convert numpy.bytes\_ to UTF-16.
+        """ Whether or not to convert numpy.bytes\\_ to UTF-16.
 
         bool
 
@@ -455,7 +455,7 @@ class Options(object):
 
         See Also
         --------
-        numpy.bytes\_
+        numpy.bytes_
         convert_numpy_str_to_utf16
 
         """
@@ -472,7 +472,7 @@ class Options(object):
 
     @property
     def convert_numpy_str_to_utf16(self):
-        """ Whether or not to convert numpy.str\_ to UTF-16.
+        """ Whether or not to convert numpy.str\\_ to UTF-16.
 
         bool
 
@@ -491,7 +491,7 @@ class Options(object):
 
         See Also
         --------
-        numpy.bytes\_
+        numpy.bytes_
         convert_numpy_str_to_utf16
 
         """
@@ -745,7 +745,6 @@ class Options(object):
         compress
         compress_size_threshold
         h5py.Group.create_dataset
-        http://www.hdfgroup.org/doc_resource/SZIP/Commercial_szip.html
 
         """
         return self._compression_algorithm
@@ -1231,7 +1230,7 @@ def writes(mdict, filename='data.h5', truncate_existing=False,
         if truncate_existing or not os.path.isfile(filename):
             f = h5py.File(filename, mode='w', userblock_size=512)
         else:
-            f = h5py.File(filename)
+            f = h5py.File(filename, mode='a')
             if options.matlab_compatible and truncate_invalid_matlab \
                     and f.userblock_size < 128:
                 f.close()
@@ -1303,16 +1302,15 @@ def writes(mdict, filename='data.h5', truncate_existing=False,
     # can be closed if any errors happen (the error is re-raised).
     f = None
     try:
-        f = h5py.File(filename)
+        f = h5py.File(filename, mode='a')
 
         # Go through each element of towrite and write them.
         for groupname, targetname, data in towrite:
             # Need to make sure groupname is a valid group in f and grab its
             # handle to pass on to the low level function.
-            if groupname not in f:
+            grp = f.get(groupname)
+            if grp is None:
                 grp = f.require_group(groupname)
-            else:
-                grp = f[groupname]
 
             # Hand off to the low level function.
             lowlevel.write_data(f, grp, targetname, data,
@@ -1497,13 +1495,13 @@ def reads(paths, filename='data.h5', options=None, **keywords):
         for groupname, targetname in toread:
             # Check that the containing group is in f and is indeed a
             # group. If it isn't an error needs to be thrown.
-            if groupname not in f \
-                    or not isinstance(f[groupname], h5py.Group):
+            grp = f.get(groupname)
+            if grp is None or not isinstance(grp, h5py.Group):
                 raise CantReadError('Could not find containing Group '
                                     + groupname + '.')
 
             # Hand off everything to the low level reader.
-            datas.append(lowlevel.read_data(f, f[groupname],
+            datas.append(lowlevel.read_data(f, grp,
                                             targetname, options))
     except:
         raise
@@ -1695,6 +1693,11 @@ def loadmat(file_name, mdict=None, appendmat=True,
     passed on. This function is modelled after the SciPy one (arguments
     not specific to this package have the same names, etc.).
 
+    Warning
+    -------
+    Variables in `variable_names` that are missing from the file do not
+    cause an exception and will just be missing from the output.
+
     Parameters
     ----------
     file_name : str
@@ -1720,7 +1723,8 @@ def loadmat(file_name, mdict=None, appendmat=True,
     -------
     dict
         Dictionary of all the variables read from the MAT file (name
-        as the key, and content as the value).
+        as the key, and content as the value). If a variable was missing
+        from the file, it will not be present here.
 
     Raises
     ------
@@ -1780,13 +1784,16 @@ def loadmat(file_name, mdict=None, appendmat=True,
                             pass
 
         else:
-            # Extract the desired fields all together and then pack them
-            # into a dictionary one by one.
-            values = reads(paths=variable_names, filename=filename,
-                           options=options)
+            # Extract the desired fields one by one, catching any errors
+            # for missing variables (so we don't fall back to
+            # scipy.io.loadmat).
             data = dict()
-            for i, name in enumerate(variable_names):
-                data[name] = values[i]
+            with h5py.File(filename, mode='r') as f:
+                for k in variable_names:
+                    try:
+                        data[k] = lowlevel.read_data(f, f, k, options)
+                    except:
+                        pass
 
         # Read all the variables, stuff them into mdict, and return it.
         if mdict is None:
