@@ -32,6 +32,19 @@ import numpy as np
 import numpy.testing as npt
 
 
+def assert_dtypes_equal(a, b):
+    # Check that two dtypes are equal, but ignorning itemsize for dtypes
+    # whose shape is 0.
+    assert isinstance(a, np.dtype)
+    assert a.shape == b.shape
+    if b.names is None:
+        assert a == b
+    else:
+        assert a.names == b.names
+        for n in b.names:
+            assert_dtypes_equal(a[n], b[n])
+
+
 def assert_equal(a, b):
     # Compares a and b for equality. If they are dictionaries, they must
     # have the same set of keys, after which they values must all be
@@ -66,7 +79,7 @@ def assert_equal(a, b):
             else:
                 assert a == b or np.all(np.isnan([a, b]))
     else:
-        assert a.dtype == b.dtype
+        assert_dtypes_equal(a.dtype, b.dtype)
         assert a.shape == b.shape
         if b.dtype.name != 'object':
             with warnings.catch_warnings():
@@ -128,6 +141,8 @@ def assert_equal_none_format(a, b):
             assert_equal_none_format(a, np.int64(b))
         else:
             assert_equal_none_format(a, np.array(b)[()])
+    elif isinstance(b, np.recarray):
+        assert_equal_none_format(a, b.view(np.ndarray))
     else:
         if b.dtype.name != 'object':
             if b.dtype.char in ('U', 'S'):
@@ -148,30 +163,58 @@ def assert_equal_none_format(a, b):
                     assert a.shape == b.shape
                     npt.assert_equal(a, b)
             else:
-                # If the original is structued, it is possible that the
-                # fields got out of order, in which case the dtype won't
-                # quite match. It will need to be checked just to make
-                # sure all pieces are there. Otherwise, the dtypes can
-                # be directly compared.
-                if b.dtype.fields is None:
-                    assert a.dtype == b.dtype
-                else:
-                    assert dict(a.dtype.fields) == dict(b.dtype.fields)
+                # Check that the dtype's shape matches.
+                assert a.dtype.shape == b.dtype.shape
                 # Now, if b.shape is just all ones, then a.shape will
                 # just be (1,). Otherwise, we need to compare the shapes
                 # directly. Also, dimensions need to be squeezed before
                 # comparison in this case.
                 assert np.prod(a.shape) == np.prod(b.shape)
-                assert a.shape == b.shape \
-                    or (np.prod(b.shape) == 1 and a.shape == (1,))
+                if a.shape != b.shape:
+                    assert np.prod(b.shape) == 1
+                    assert a.shape == (1, )
                 if np.prod(a.shape) == 1:
                     a = np.squeeze(a)
                     b = np.squeeze(b)
-                with warnings.catch_warnings():
-                    warnings.simplefilter('ignore', RuntimeWarning)
-                    npt.assert_equal(a, b)
+
+                # If there was a null in the dtype or the dtype of one
+                # of its fields (or subfields) has a 0 in its shape,
+                # then it was written as a Group so the field order
+                # could have changed.
+                has_zero_shape = False
+                if b.dtype.names is not None:
+                    parts = [b.dtype]
+                    while 0 != len(parts):
+                        part = parts.pop()
+                        if 0 in part.shape:
+                            has_zero_shape = True
+                        if part.names is not None:
+                            parts.extend([v[0] for v
+                                          in part.fields.values()])
+                        if part.base != part:
+                            parts.append(part.base)
+                if b.dtype.names is not None \
+                        and ('\\x00' in str(b.dtype) \
+                        or has_zero_shape):
+                    assert a.shape == b.shape
+                    assert set(a.dtype.names) == set(b.dtype.names)
+                    for n in b.dtype.names:
+                        assert_equal_none_format(a[n], b[n])
+                else:
+                    assert a.dtype == b.dtype
+                    with warnings.catch_warnings():
+                        warnings.simplefilter('ignore', RuntimeWarning)
+                        npt.assert_equal(a, b)
         else:
-            assert a.dtype == b.dtype
+            # If the original is structued, it is possible that the
+            # fields got out of order, in which case the dtype won't
+            # quite match. It will need to be checked just to make sure
+            # all pieces are there. Otherwise, the dtypes can be
+            # directly compared.
+            if b.dtype.fields is None:
+                assert a.dtype == b.dtype
+            else:
+                assert dict(a.dtype.fields) == dict(b.dtype.fields)
             assert a.shape == b.shape
             for index, x in np.ndenumerate(a):
                 assert_equal_none_format(a[index], b[index])
